Package: MOSim
Title: Multi-Omics Simulation (MOSim)
Version: 2.6.0
Authors@R: c(person("Carolina", "Monzó", email = "carolmonzoc@gmail.com", role = c("aut")), person("Carlos", "Martínez", email = "cmarmir@gmail.com", role = "aut"), person("Sonia", "Tarazona", email = "sotacam@gmail.com", role = c("cre", "aut")))
Description: MOSim package simulates multi-omic experiments that mimic regulatory mechanisms within the cell, allowing flexible experimental design including time course and multiple groups.
Encoding: UTF-8
Depends: R (>= 4.2.0)
License: GPL-3
LazyData: false
biocViews: Software, TimeCourse, ExperimentalDesign, RNASeq
BugReports: https://github.com/ConesaLab/MOSim/issues
URL: https://github.com/ConesaLab/MOSim
Imports: HiddenMarkov, zoo, IRanges, S4Vectors, dplyr, ggplot2,
        lazyeval, matrixStats, methods, rlang, stringi, stringr, scran,
        Seurat, Signac, edgeR, Rcpp
Suggests: testthat, knitr, rmarkdown, codetools, BiocStyle, stats,
        utils, purrr, scales, tibble, tidyr, Biobase, scater,
        SingleCellExperiment, decor, markdown, Rsamtools, igraph,
        leiden, bluster
Collate: 'AllClass.R' 'AllGeneric.R' 'Simulator.R' 'SimulatorRegion.R'
        'ChIP-seq.R' 'DNase-seq.R' 'MOSim-package.R' 'functions.R'
        'Simulation.R' 'MOSim.R' 'RNA-seq.R' 'data.R'
        'simulate_WGBS_functions.R' 'methyl-seq.R' 'miRNA-seq.R'
        'sc_MOSim.R' 'sc_coexpression.R' 'sparsim_functions.R' 'zzz.R'
RoxygenNote: 7.3.2
VignetteBuilder: knitr
LinkingTo: cpp11, Rcpp
git_url: https://git.bioconductor.org/packages/MOSim
git_branch: RELEASE_3_22
git_last_commit: 3eff748
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29
NeedsCompilation: yes
Packaged: 2025-10-30 05:18:41 UTC; biocbuild
Author: Carolina Monzó [aut],
  Carlos Martínez [aut],
  Sonia Tarazona [cre, aut]
Maintainer: Sonia Tarazona <sotacam@gmail.com>
